-- Manually returns the total of the dice rolls from a damage or healing effect.
-- Values for luckMode:
-- 0: Luck won't affect die rolls.
-- 1: Weapon damage: Source's luck and minimum damage stat are added to each die roll.
-- 2: Spell damage: Target's luck is subtracted from each die roll.
-- 3: Healing: Target's luck is added to each die roll.
function EEex_RollEffectDice(targetID, sourceID, dicenumber, dicesize, luckMode)
	if dicenumber == 0 or dicesize == 0 then return 0 end
	local total = 0
	local sourceLuck = EEex_GetActorStat(sourceID, 32)
	local sourceWeaponLuck = EEex_GetActorStat(sourceID, 145)
	local sourceSpellMinimum = EEex_GetActorStat(sourceID, 621)
	local targetLuck = EEex_GetActorStat(targetID, 32)
	for i = 1, dicenumber, 1 do
		local roll = math.random(dicesize)
		if luckMode == 1 then
			roll = roll + sourceLuck + sourceWeaponLuck
		elseif luckMode == 2 then
			roll = roll - targetLuck
			if roll <= sourceSpellMinimum then
				roll = sourceSpellMinimum + 1
			end
		elseif luckMode == 3 then
			roll = roll + targetLuck
			if roll <= sourceSpellMinimum then
				roll = sourceSpellMinimum + 1
			end
		end
		if roll > dicesize then
			roll = dicesize
		elseif roll < 1 then
			roll = 1
		end
		total = total + roll
	end
	return total
end

--[[
To use the EXDAMAGE function, create an opcode 402 effect in an item or spell, set the resource to EXDAMAGE (all capitals),
 set the timing to instant, limited and the duration to 0, and choose parameters.
For an example of this function in use, look at EXFLAMEB.ITM.

The EXDAMAGE function deals damage to the target. The main use of it is to put it on a weapon that should deal non-physical
 damage, such as the Flame Blade. The function can add bonuses to the damage dealt based on the character's Strength, proficiencies,
 general weapon damage bonuses, melee damage bonuses, missile damage bonuses, or fist damage bonuses. This can't be done simply
 by applying a damage effect normally.

parameter1 - The first byte determines the damage, the second byte determines the dice size, the third byte determines the dice number,
 and the fourth byte determines the proficiency used. For example, if the effect is from a bastard sword and should do 2d4+3 damage,
 parameter1 should be this:
 0x59020403
 0x59 is the bastard sword proficiency number, 0x2 is the dice number, 0x4 is the dice size, and 0x3 is the damage bonus.
 If a proficiency is not specified, it doesn't give a damage bonus based on proficiency.

parameter2 - It's the same as parameter2 on the damage opcode: the first two bytes determine whether to just deal damage or set HP or whatever.
 The last two bytes determine the damage type. If you simply want to deal fire damage, parameter2 would be 0x80000 (look at DMGTYPE.IDS).

savingthrow - This function uses several extra bits on this parameter:
Bit 16: If set, the source character's Strength bonus is added to the damage.
Bit 17: If set, the damage is treated as the base damage of a melee weapon, so it gets damage bonuses from opcodes 73 and 285.
Bit 18: If set, the damage is treated as the base damage of a missile weapon, so it gets damage bonuses from opcodes 73 and 286.
Bit 19: If set, the damage is treated as the base damage of a fist weapon, so it gets damage bonuses from opcodes 73 and 289.
If more than one of bits 17, 18, and 19 are set, opcode 73 damage bonuses are not applied multiple times. Also, if at least one
 of those three bits are set, the minimum damage of each die will be increased based on the source character's Luck and opcode 250 bonuses.
 If none of those three bits are set, the maximum damage of each die is decreased based on the target character's Luck bonuses.
Bit 26: If set, there is a Save vs. Spell against the damage (set this bit instead of bit 0 if you're using "Save for half")
Bit 27: If set, there is a Save vs. Breath against the damage (set this bit instead of bit 1 if you're using "Save for half")
Bit 28: If set, there is a Save vs. Death against the damage (set this bit instead of bit 2 if you're using "Save for half")
Bit 29: If set, there is a Save vs. Wand against the damage (set this bit instead of bit 3 if you're using "Save for half")
Bit 30: If set, there is a Save vs. Polymorph against the damage (set this bit instead of bit 4 if you're using "Save for half")
special - It's the same as special on the damage opcode.
--]]

ex_proficiency_damage = {[0] = 0, [1] = 0, [2] = 2, [3] = 3, [4] = 4, [5] = 5}
ex_strength_damage = {[0] = -20, [1] = -4, [2] = -2, [3] = -1, [4] = -1, [5] = -1, [6] = 0, [7] = 0, [8] = 0, [9] = 0, [10] = 0, [11] = 0, [12] = 0, [13] = 0, [14] = 0, [15] = 0, [16] = 1, [17] = 1, [18] = 2, [19] = 7, [20] = 8, [21] = 9, [22] = 10, [23] = 11, [24] = 12, [25] = 14}
function EXDAMAGE(effectData, creatureData)
	local sourceID = EEex_ReadDword(effectData + 0x10C)
	local targetID = EEex_ReadDword(creatureData + 0x34)
	local damage = EEex_ReadByte(effectData + 0x18, 0x0)
	local dicesize = EEex_ReadByte(effectData + 0x19, 0x0)
	local dicenumber = EEex_ReadByte(effectData + 0x1A, 0x0)
	local proficiency = EEex_ReadByte(effectData + 0x1B, 0x0)
	local parameter2 = EEex_ReadDword(effectData + 0x1C)
	local savingthrow = EEex_ReadDword(effectData + 0x3C)
	local special = EEex_ReadDword(effectData + 0x44)
	local restype = EEex_ReadDword(effectData + 0x8C)
	local vvcresource = EEex_ReadLString(effectData + 0x6C, 8)
	local resource2 = EEex_ReadLString(effectData + 0x74, 8)
	local launcherRES = ""
	local parent_resource = EEex_ReadLString(effectData + 0x90, 8)
	local casterlvl = EEex_ReadDword(effectData + 0xC4)
	if bit32.band(savingthrow, 0x40000) > 0 then
		local itemData = EEex_DemandResData(parent_resource, "ITM")
		if itemData > 1000 then
			local itemType = EEex_ReadWord(itemData + 0x1C, 0x0)
			
			if itemType == 5 or itemType == 14 or itemType == 31 then
				local found_it = false
				EEex_IterateActorEffects(sourceID, function(eData)
					local the_timing = EEex_ReadDword(eData + 0x24)
					local the_sourceslot = EEex_ReadDword(eData + 0xA4)
					if the_timing == 2 and the_sourceslot >= 35 and the_sourceslot <= 38 and found_it == false then
						found_it = true
						launcherRES = EEex_ReadLString(eData + 0x94, 8)
						local launcherData = EEex_DemandResData(launcherRES, "ITM")
						if launcherData > 1000 then
							local launcherType = EEex_ReadWord(launcherData + 0x1C, 0x0)
							if (itemType == 5 and launcherType == 15) or (itemType == 14 and launcherType == 18) or (itemType == 31 and launcherType == 27) then
								proficiency = EEex_ReadByte(launcherData + 0x31, 0x0)
								damage = damage + EEex_ReadSignedWord(launcherData + 0x8C, 0x0)
							end
						end
					end
				end)
			end
		end
		if proficiency == 0 then
			if itemType == 5 then
				if EEex_GetActorStat(sourceID, 104) > EEex_GetActorStat(sourceID, 105) then
					proficiency = 104
				else
					proficiency = 105
				end
			elseif itemType == 14 then
				proficiency = 107
			elseif itemType == 31 then
				proficiency = 103
			end
		end
		damage = damage + EEex_GetActorStat(sourceID, 168)
	end
	if proficiency > 0 and ex_proficiency_damage[EEex_GetActorStat(sourceID, proficiency)] ~= nil then
		damage = damage + ex_proficiency_damage[EEex_GetActorStat(sourceID, proficiency)]
	end
	if bit32.band(savingthrow, 0x4000000) > 0 then
		savingthrow = bit32.band(savingthrow, 0x1)
	end
	if bit32.band(savingthrow, 0x8000000) > 0 then
		savingthrow = bit32.band(savingthrow, 0x2)
	end
	if bit32.band(savingthrow, 0x10000000) > 0 then
		savingthrow = bit32.band(savingthrow, 0x4)
	end
	if bit32.band(savingthrow, 0x20000000) > 0 then
		savingthrow = bit32.band(savingthrow, 0x8)
	end
	if bit32.band(savingthrow, 0x40000000) > 0 then
		savingthrow = bit32.band(savingthrow, 0x10)
	end
	if bit32.band(savingthrow, 0x10000) > 0 then
		local strength = EEex_GetActorStat(sourceID, 36)
		if strength < 0 then
			strength = 0
		elseif strength > 25 then
			strength = 25
		end
		damage = damage + ex_strength_damage[strength]
		if strength == 18 then
			local exStrength = EEex_GetActorStat(sourceID, 37)
			if exStrength >= 1 then
				damage = damage + 1
			end
			if exStrength >= 76 then
				damage = damage + 1
			end
			if exStrength >= 91 then
				damage = damage + 1
			end
			if exStrength >= 100 then
				damage = damage + 1
			end
		end
	end
	if bit32.band(savingthrow, 0x20000) > 0 then
		damage = damage + EEex_GetActorStat(sourceID, 167)
	end
	if bit32.band(savingthrow, 0x40000) > 0 then
		damage = damage + EEex_GetActorStat(sourceID, 168)
	end
	if bit32.band(savingthrow, 0x80000) > 0 then
		damage = damage + EEex_GetActorStat(sourceID, 171)
	end
	local luck = 0
	if bit32.band(savingthrow, 0x20000) > 0 or bit32.band(savingthrow, 0x40000) > 0 or bit32.band(savingthrow, 0x80000) > 0 then
		damage = damage + EEex_RollEffectDice(targetID, sourceID, dicenumber, dicesize, 1)
		damage = damage + EEex_GetActorStat(sourceID, 50)
		restype = 0
		vvcresource = parent_resource
		resource2 = launcherRES
		parent_resource = "EEEX_DAM"

	else
		restype = 1
		damage = damage + EEex_RollEffectDice(targetID, sourceID, dicenumber, dicesize, 2)
	end
	EEex_ApplyEffectToActor(targetID, {
["opcode"] = 12,
["target"] = 2,
["timing"] = 1,
["parameter1"] = damage,
["parameter2"] = parameter2,
["savingthrow"] = savingthrow,
["special"] = special,
["vvcresource"] = vvcresource,
["resource2"] = resource2,
["restype"] = restype,
["parent_resource"] = parent_resource,
["source_target"] = targetID,
["source_id"] = sourceID
})
end

EEex_AddScreenEffectsGlobal("MEREAVER", function(effectData, creatureData)
	local internal_flags = EEex_ReadDword(effectData + 0xC8)
	local opcode = EEex_ReadDword(effectData + 0xC)
	local parameter2 = EEex_ReadDword(effectData + 0x1C)
	local parent_resource = EEex_ReadLString(effectData + 0x90, 8)
	if opcode == 12 and (parent_resource == "" or parent_resource == "EEEX_DAM") and (parameter2 == 0 or bit32.band(parameter2, 0x9900000) > 0) then
		local weaponRES = EEex_ReadLString(effectData + 0x6C, 8)
		local weaponData = EEex_DemandResData(weaponRES, "ITM")
		if weaponData > 1000 and bit32.band(EEex_ReadDword(weaponData + 0x18), 0x200000) > 0 then
			return true
		end
	end
end)